<?php declare(strict_types=1);

namespace TurnstileReMaster\Storefront\Framework\Captcha;

use Psr\Http\Client\ClientExceptionInterface;
use Psr\Http\Client\ClientInterface; // Use PSR-18 ClientInterface
use Psr\Log\LoggerInterface;
use Shopware\Storefront\Framework\Captcha\AbstractCaptcha;
use Symfony\Component\HttpFoundation\Request;
use Throwable;

final class CloudFlareTurnstile extends AbstractCaptcha
{
    public const CAPTCHA_NAME = 'cloudFlareTurnstile';
    public const CAPTCHA_REQUEST_PARAMETER = 'cf-turnstile-response';
    private const CLOUDFLARE_CAPTCHA_VERIFY_ENDPOINT = 'https://challenges.cloudflare.com/turnstile/v0/siteverify';

    private ClientInterface $client;
    private LoggerInterface $logger;

    public function __construct(
        ClientInterface $client, // Inject PSR-18 client
        LoggerInterface $logger
    ) {
        $this->client = $client;
        $this->logger = $logger;
    }

    public function isValid(Request $request, array $captchaConfig): bool
    {
        $turnstileResponse = $request->request->get(self::CAPTCHA_REQUEST_PARAMETER);

        if (!\is_string($turnstileResponse) || $turnstileResponse === '') {
            $this->logger->debug('Cloudflare Turnstile token missing in request.');
            return false;
        }

        $secretKey = $captchaConfig['config']['secretKey'] ?? null;

        if (!\is_string($secretKey) || $secretKey === '') {
            $this->logger->error('Cloudflare Turnstile secret key is missing or not configured.');
            return false;
        }

        try {
            // Prepare request data for PSR-18 client
            $options = [
                'body' => http_build_query([ // Send as form data
                    'secret' => $secretKey,
                    'response' => $turnstileResponse,
                    'remoteip' => $request->getClientIp(),
                ]),
                'headers' => [
                    'Content-Type' => 'application/x-www-form-urlencoded',
                ],
                //TODO Timeout handling might need custom setup depending on the client implementation
            ];

            $apiRequest = $this->client->createRequest('POST', self::CLOUDFLARE_CAPTCHA_VERIFY_ENDPOINT);
            foreach ($options['headers'] as $key => $value) {
                $apiRequest = $apiRequest->withHeader($key, $value);
            }
            $apiRequest->getBody()->write($options['body']);


            $response = $this->client->sendRequest($apiRequest);

            $responseBody = $response->getBody()->getContents();
            $responseData = json_decode($responseBody, true);

            if (json_last_error() !== JSON_ERROR_NONE) {
                $this->logger->warning('Failed to decode Cloudflare Turnstile verify response.', ['response_body' => $responseBody]);
                return false;
            }

            if (!isset($responseData['success'])) {
                $this->logger->warning('Cloudflare Turnstile verify response missing "success" key.', ['response_data' => $responseData]);
                return false;
            }

            if ($responseData['success'] === true) {
                $this->logger->debug('Cloudflare Turnstile validation successful.');
                return true;
            }

            $errorCodes = $responseData['error-codes'] ?? [];
            $this->logger->info('Cloudflare Turnstile validation failed.', [
                'success' => false,
                'error-codes' => $errorCodes,
                'clientIp' => $request->getClientIp()
            ]);
            return false;

        } catch (ClientExceptionInterface $e) {
            $this->logger->error('HTTP error during Cloudflare Turnstile verification.', ['exception' => $e->getMessage()]);
            return false;
        } catch (Throwable $e) {
            $this->logger->error('Unexpected error during Cloudflare Turnstile verification.', ['exception' => $e->getMessage()]);
            return false;
        }
    }

    public function getName(): string
    {
        return self::CAPTCHA_NAME;
    }
}